// Test programme to verify that array operations are faster than classes
// Keermalec, 30 September 2003

#include <stdio.h>           				// Standard C/C++ Input-Output, FILE, fopen(), sprintf(), fwrite(), fclose()
#include <windows.h>         				// Standard Header For MSWindows Applications
#include <math.h>							// Math functions such as sqrt(), sin(), cos()
#include <time.h>							// for clock()


#define M_NUMVALUES 1000000					// Number of tests to run


// Our standard array, simple isn't it?
typedef float M3float[3];


// Our xyz vector class
class Mvector
{
	public:
		float x, y, z;
//		Mvector (float a, float b, float c) { x = a; y = b; z = c; }
		Mvector (float a, float b, float c) : x(a), y(b), z(c) {}
		Mvector inline operator+(Mvector &a) { return Mvector(x + a.x, y + a.y, z + a.z); };
		Mvector () {}					
		~Mvector () {}				
};


// Array addiion
void arrAdd(M3float a, M3float b, M3float &c)
{
	c[0] = a[0] + b[0];
	c[1] = a[1] + b[1];
	c[2] = a[2] + b[2];
}


// The following funtion generates a random float from rangelow to rangehigh
float mRandGen(unsigned int seed, float rangelow, float rangehigh)
{
	srand(seed);							// defines seed to be used in rand()
	return ((rand()*(rangehigh-rangelow))/32768)+rangelow;
}


int main (void)
{
	printf("\nProving that arrays are faster than classes,\n");
	printf("\nBy Keermalec, 30 september 2003\n");
	printf("\n----------------------------------\n");

	// Some useful variables
	unsigned int RandSeed, a, b;
	float startTime, endTime, vecTime, arrTime;
	
	// 3 sets of arrays and 3 sets of vectors
	M3float *array_1;
	array_1 = (M3float*) calloc (M_NUMVALUES, sizeof M3float);
	M3float *array_2;
	array_2 = (M3float*) calloc (M_NUMVALUES, sizeof M3float);
	M3float *array_3;
	array_3 = (M3float*) calloc (M_NUMVALUES, sizeof M3float);
	Mvector *vector_1;
	vector_1 = (Mvector*) calloc (M_NUMVALUES, sizeof Mvector);
	Mvector *vector_2;
	vector_2 = (Mvector*) calloc (M_NUMVALUES, sizeof Mvector);
	Mvector *vector_3;
	vector_3 = (Mvector*) calloc (M_NUMVALUES, sizeof Mvector);
	
	// Create the random seed
	RandSeed = mRandGen(((unsigned)time( NULL )),1,32768);
	
	// Load random values
	for ( a= 0; a < M_NUMVALUES; a++ )
	{
		for ( b = 0; b < 3; b++ )
		{
			array_1[a][b] = mRandGen((a+b)*RandSeed*rand(),-100.0f,100.0f);
			array_2[a][b] = mRandGen((a+b)*RandSeed*rand(),-100.0f,100.0f);
		}
		vector_1[a].x = array_1[a][0];
		vector_1[a].y = array_1[a][1];
		vector_1[a].z = array_1[a][2];
		vector_2[a].x = array_2[a][0];
		vector_2[a].y = array_2[a][1];
		vector_2[a].z = array_2[a][2];
	}
	printf("\n%i random vectors and arrays created\n", M_NUMVALUES);

	// Now do some additions

	// Array addition
	startTime = (float)clock()/CLOCKS_PER_SEC;
	for ( b = 0; b < M_NUMVALUES; b++ )
	{
		arrAdd(array_1[b], array_2[b], array_3[b]);
	}
	endTime = (float)clock()/CLOCKS_PER_SEC;
	arrTime = endTime - startTime;
	printf("\nAdding %i arrays: % 5.6f seconds\n", M_NUMVALUES, arrTime);

	// Mvector addition
	startTime = (float)clock()/CLOCKS_PER_SEC;
	for ( b = 0; b < M_NUMVALUES; b++ )
	{
		vector_3[b] = vector_1[b] + vector_2[b];
	}
	endTime = (float)clock()/CLOCKS_PER_SEC;
	vecTime = endTime - startTime;
	printf("Adding %i vectors: % 5.6f seconds\n", M_NUMVALUES, vecTime);

	printf("\nCalculating %i additions of random vectors took:", M_NUMVALUES);
	printf("\n\n% 5.3f seconds using the xyz vector class", vecTime);
	printf("\n% 5.3f seconds using arrays", arrTime);
	printf("\n\nTherefore arrays are % 5.3f x faster than xyz classes.", vecTime/arrTime);
	printf("\n\nThanks to Brother Bob and _DarkWing_ for nagging me until I wrote this\n");
	printf("\n----------------------------------\n");
	return 0;
}